<?php

namespace  Onestartup\FacturaDigital\Lib;

use GuzzleHttp\Client as GuzzleClient;

class Factura
{

	private $concepto;
	private $listaConceptos;
	private $receptor;
	private $impuesto;
	private $emisor;

	private $Serie;
	private $Folio;
	private $Fecha;
	private $FormaPago;
	private $CondicionesDePago;
	private $SubTotal;
	private $Descuento;
	private $Moneda;
	private $TipoCambio;
	private $Total;
	private $TipoDeComprobante;
	private $MetodoPago;
	private $LugarExpedicion;
	private $LeyendaFolio;

	private $data;

	public function __construct(
		FacturaConcepto $concepto, 
		FacturaConceptoLista $listaConceptos, 
		FacturaReceptor $receptor, 
		FacturaImpuesto $impuesto,
		FacturaEmisor $emisor)
	{
		$this->concepto = $concepto;
		$this->listaConceptos = $listaConceptos;
		$this->receptor = $receptor;
		$this->impuesto = $impuesto;
		$this->emisor = $emisor;
	}


    /**
     * @var FacturaConcepto $concepto
     */
    
    public function concepto()
    {
    	return $this->concepto;
    }


    /**
     * @var FacturaConceptoLista $listaConceptos
     */
    public function listaConceptos()
    {
    	return $this->listaConceptos;
    }


    /**
     * @var FacturaReceptor $receptor
     */
    public function receptor()
    {
    	return $this->receptor;
    }


    /**
     * @var FacturaImpuesto $impuesto
     */
    public function impuesto()
    {
    	return $this->impuesto;
    }


     /**
     * @var FacturaEmisor $emisro
     */
     public function emisor()
     {
     	return $this->emisor;
     }



	/**
	 * Consulta el numero de creditos restantes
	 * @return @array [mensaje, creditos, codigo] respuesta de la api
	 */
	public function creditos()
	{
		
		$client = new GuzzleClient();
		$res = $client->request('POST', $this->base_url().'/cfdi/creditos', [
			'headers' => $this->credenciales()
		]);

		$respuesta = $res->getBody()->getContents();
		$data = json_decode($respuesta, true);

		return $data;
	}

	public function enviar()
	{
		$res = null;
        $data_cfdi = [];

		try {

			$data = $this->getData();
            $part_headers = ['Accept' => 'application/json', 'jsoncfdi' =>  json_encode($data) ];
			$headers = array_merge( $this->credenciales(), $part_headers);

			$randomkey = rand(5, 9999999999999999);


			$client = new GuzzleClient();
			$res = $client->request('POST', $this->base_url().'/cfdi/generar/'.$randomkey, [
				'headers' => $headers
			]);

			if ($res) {
				$respuesta = $res->getBody()->getContents();
                $data = json_decode($respuesta, true);
                $data_cfdi = $data;
			} else {
				return "Error api";
			}

		} catch (\GuzzleHttp\Exception\ClientException $e) {
			 //dd($e->getResponse()->getBody()->getContents());
             $error = json_decode($e->getResponse()->getBody()->getContents());
             $error = (array)$error;
             $data_cfdi = $error;
		}

        return $data_cfdi;
	}


	/**
	 * Obtiene la url base de la api
	 * @return @string URL
	 */
	protected function base_url()
	{
		$url_base = 'http://app.facturadigital.com.mx/api';

		return $url_base ;
	}


	/**
	 * Obtiene las cebeceras necerarias para el inicio de sesion
	 * @return @array con usuario y contraseña
	 */
	protected function credenciales()
	{
		$headers = array('api-usuario' 	=> env('USER_FACTURA_DIGITAL'), 
			'api-password' => env('PASSWORD_FACTURA_DIGITAL'));

		return $headers;
	}

    public function precioSinIva($precioConIva)
    {
        $data = [];
        $precioSin = ($precioConIva * 100) / 116;
        $iva = $precioSin * 0.16000;

        $data = ['precioSin' => $precioSin, 'iva' => $iva];


        return $data;
    }

	/**
	 * Devuelve el arreglo con los datos correctos
	 * @return Array
	 */
	public function getData()
	{
		return $this->data;
	}


    public function test()
    {
        # llenamos los datos de nuestro CFDI
    # crearemos un xml de prueba
    $d = array();

    #$d['printxml'] = true; # nos permite imprimir el xml generado para debug

    # datos basicos SAT
    $d['Serie']             = 'F';
    $d['Folio']             = '987750';  #'101';
    $d['Fecha']             = 'AUTO';
    $d['FormaPago']         = '01';
    $d['CondicionesDePago'] = 'CONDICIONES';
    $d['SubTotal']          = '200.00';
    $d['Descuento']         = '50.00'; # o bien: null
    $d['Moneda']            = 'MXN';
    $d['TipoCambio']        = 1;
    $d['Total']             = '174.00';
    $d['TipoDeComprobante'] = 'I';
    $d['MetodoPago']        = 'PUE';
    $d['LugarExpedicion']   = '67150';

    # opciones de personalización (opcionales)
    $d['LeyendaFolio']      = "FACTURA"; # leyenda opcional para poner a lado del folio: FACTURA, RECIBO, NOTA DE CREDITO, ETC.

    # codigo de confirmación PAC para cfdis mayores a $20 millones
    # $d['Confirmacion'] = null;

    # CFDIs relacionados
    # $d['CfdiRelacionados']['TipoRelacion'] = null;
    # $d['CfdiRelacionados'][0]['UUID'] = null;

    # Regimen fiscal del emisor ligado al tipo de operaciones que representa este CFDI
    $d['Emisor']['RegimenFiscal'] = '615'; # ver catálogo del SAT

    # Datos del receptor
    $d['Receptor']['Rfc'] = 'NDI120326HF5';
    $d['Receptor']['Nombre'] = 'Novatech Digital SA de CV';
    # $d['Receptor']['ResidenciaFiscal'] = 'MEX'; # solo se usa cuando el receptor no esté dado de alta en el SAT
    $d['Receptor']['NumRegIdTrib'] = ''; # para extranjeros
    $d['Receptor']['UsoCFDI'] = 'G03'; # uso que le dará el cliente al cfdi

    # Receptor -> Domicilio (OPCIONAL)
    $d["Receptor"]["Calle"] = "Palmas";
    $d["Receptor"]["NoExt"] = "9810";
    #$d["Receptor"]["NoInt"] = null;
    $d["Receptor"]["Colonia"] = "Anahuac";
    #$d["Receptor"]["Localidad"] = null;
    #$d["Receptor"]["Referencia"] = null;
    $d["Receptor"]["Municipio"] = "Apodaca";
    $d["Receptor"]["Estado"] = "Nuevo Leon";
    $d["Receptor"]["Pais"] = "México";
    $d["Receptor"]["CodigoPostal"] = "67349";

    # >> conceptos <<
    # concepto 1
    $d['Conceptos'][0]['ClaveProdServ'] = '01010101';
    $d['Conceptos'][0]['NoIdentificacion'] = '01'; #codigo interno o SKU, GTIN, codigo de barras, etc.
    $d['Conceptos'][0]['Cantidad'] = 1.00;
    $d['Conceptos'][0]['ClaveUnidad'] = 'KGM'; # Clave SAT
    $d['Conceptos'][0]['Unidad'] = 'Kilo'; # Unidad de Medida
    $d['Conceptos'][0]['Descripcion'] = 'PANTALLA LCD SHARP'; #maximo 1000 caracteres
    $d['Conceptos'][0]['ValorUnitario'] = '100.00';
    $d['Conceptos'][0]['Importe'] = '100.00';
    $d['Conceptos'][0]['Descuento'] = '25.00'; # no se permiten valores negativos

    # concepto 1 -> impuestos
    $d['Conceptos'][0]['Impuestos']['Traslados'][0]['Base']         = '75.00';
    $d['Conceptos'][0]['Impuestos']['Traslados'][0]['Impuesto']     = '002';
    $d['Conceptos'][0]['Impuestos']['Traslados'][0]['TipoFactor']   = 'Tasa';
    $d['Conceptos'][0]['Impuestos']['Traslados'][0]['TasaOCuota']   = '0.160000';
    $d['Conceptos'][0]['Impuestos']['Traslados'][0]['Importe']      = '12.00';






    $d['Conceptos'][1]['ClaveProdServ'] = '01010101';
    $d['Conceptos'][1]['NoIdentificacion'] = '01'; #codigo interno o SKU, GTIN, codigo de barras, etc.
    $d['Conceptos'][1]['Cantidad'] = 1.00;
    $d['Conceptos'][1]['ClaveUnidad'] = 'KGM'; # Clave SAT
    $d['Conceptos'][1]['Unidad'] = 'PZA'; # Unidad de Medida
    $d['Conceptos'][1]['Descripcion'] = 'MEMORIA USB'; #maximo 1000 caracteres
    $d['Conceptos'][1]['ValorUnitario'] = '100.00';
    $d['Conceptos'][1]['Importe'] = '100.00';
    $d['Conceptos'][1]['Descuento'] = '25.00';


    $d['Conceptos'][1]['Impuestos']['Traslados'][0]['Base']         = '75.00';
    $d['Conceptos'][1]['Impuestos']['Traslados'][0]['Impuesto']     = '002';
    $d['Conceptos'][1]['Impuestos']['Traslados'][0]['TipoFactor']   = 'Tasa';
    $d['Conceptos'][1]['Impuestos']['Traslados'][0]['TasaOCuota']   = '0.160000';
    $d['Conceptos'][1]['Impuestos']['Traslados'][0]['Importe']      = '12.00';

    /*
    $d['Concepto'][0]['Impuestos']['Retenciones'][0]['Base'] = 1;
    $d['Concepto'][0]['Impuestos']['Retenciones'][0]['Impuesto'] = '002';
    $d['Concepto'][0]['Impuestos']['Retenciones'][0]['TipoFactor'] = 'Tasa';
    $d['Concepto'][0]['Impuestos']['Retenciones'][0]['TasaOCuota'] = 16.000;
    $d['Concepto'][0]['Impuestos']['Retenciones'][0]['Importe'] = 150.00;
    */

    /*
    $d['Concepto'][0]['InformacionAduanera'][0]['NumeroPedimento'] = '10 47 4344 8783676';
    $d['Concepto'][0]['InformacionAduanera'][1]['NumeroPedimento'] = '10 47 6564 1197423';
    */

    # concepto 2
    #$d['Conceptos'][1]['ClaveProdServ'] = '01010101';
    #$d['Conceptos'][1]['NoIdentificacion'] = '01'; #codigo interno o SKU, GTIN, codigo de barras, etc.
    #$d['Conceptos'][1]['Cantidad'] = 1.00;
    #$d['Conceptos'][1]['ClaveUnidad'] = 'F52';
    #$d['Conceptos'][1]['Unidad'] = 'TONELADA';
    #$d['Conceptos'][1]['Descripcion'] = 'ESTRUCTURA DE ACERO INOXIDABLE SKX478'; #maximo 1000 caracteres
    #$d['Conceptos'][1]['ValorUnitario'] = 500.00;
    #$d['Conceptos'][1]['Importe'] = 500.00; # solo valores positivos si el tipo de comprobante es "I", "E", "N". Si es "T" puede ser mayor o igual a cero. Si es "P" (Pago), debe ser cero forzosamente.
    #Conceptos$d['Concepto'][1]['Descuento'] = null; # no se permiten valores negativos

    # concepto 2 -> impuestos
    /*
    $d['Concepto'][1]['Impuestos']['Traslados'][0]['Base'] = 1;
    $d['Concepto'][1]['Impuestos']['Traslados'][0]['Impuesto'] = 'IVA';
    $d['Concepto'][1]['Impuestos']['Traslados'][0]['TipoFactor'] = 'Tasa';
    $d['Concepto'][1]['Impuestos']['Traslados'][0]['TasaOCuota'] = 1;
    $d['Concepto'][1]['Impuestos']['Traslados'][0]['Importe'] = 85.00;

    $d['Concepto'][1]['Impuestos']['Traslados'][1]['Base'] = 2;
    $d['Concepto'][1]['Impuestos']['Traslados'][1]['Impuesto'] = 'ISR';
    $d['Concepto'][1]['Impuestos']['Traslados'][1]['TipoFactor'] = 'Tasa';
    $d['Concepto'][1]['Impuestos']['Traslados'][1]['TasaOCuota'] = 1;
    $d['Concepto'][1]['Impuestos']['Traslados'][1]['Importe'] = 240.00;

    $d['Concepto'][1]['Impuestos']['Traslados'][2]['Base'] = 2;
    $d['Concepto'][1]['Impuestos']['Traslados'][2]['Impuesto'] = 'IEPS';
    $d['Concepto'][1]['Impuestos']['Traslados'][2]['TipoFactor'] = 'Tasa';
    $d['Concepto'][1]['Impuestos']['Traslados'][2]['TasaOCuota'] = 1;
    $d['Concepto'][1]['Impuestos']['Traslados'][2]['Importe'] = 150.00;

    $d['Concepto'][1]['Impuestos']['Retenciones'][0]['Base'] = 2;
    $d['Concepto'][1]['Impuestos']['Retenciones'][0]['Impuesto'] = 'IVA';
    $d['Concepto'][1]['Impuestos']['Retenciones'][0]['TipoFactor'] = 'Tasa';
    $d['Concepto'][1]['Impuestos']['Retenciones'][0]['TasaOCuota'] = 1;
    $d['Concepto'][1]['Impuestos']['Retenciones'][0]['Importe'] = 150.00;

    $d['Concepto'][1]['InformacionAduanera'][0]['NumeroPedimento'] = '10 47 3807 8003832';
    $d['Concepto'][1]['InformacionAduanera'][1]['NumeroPedimento'] = '10 47 3807 4322343';

    $d['Concepto'][1]['CuentaPredial']['Numero'] = '4328942809'; */

    # Impuestos
    #$d['Impuestos']['TotalImpuestosRetenidos']     = 0.000000;
    $d['Impuestos']['TotalImpuestosTrasladados']    = '24.00';

    # Definimos a detalle las retenciones
    #$d['Impuestos']['Retenciones'][0]['Impuesto']  = '001'; # 001=ISR, 002=IVA, 003=IEPS
    #$d['Impuestos']['Retenciones'][0]['Importe']   = 0.00;

    # Definimos a detalle los traslados
    $d['Impuestos']['Traslados'][0]['Impuesto']     = '002'; # 001=ISR, 002=IVA, 003=IEPS
    $d['Impuestos']['Traslados'][0]['TipoFactor']   = 'Tasa';
    $d['Impuestos']['Traslados'][0]['TasaOCuota']   = '0.160000'; # 16%
    $d['Impuestos']['Traslados'][0]['Importe']      = '24.00'; # Monto

    return $d;
    }


    /**
     * @return mixed
     */
    public function getSerie()
    {
    	return $this->Serie;
    }

    /**
     * @param mixed $Serie
     *
     * @return self
     */
    public function setSerie($Serie)
    {
    	$this->Serie = $Serie;
    	$this->data['Serie'] = $Serie;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getFolio()
    {
    	return $this->Folio;
    }

    /**
     * @param mixed $Folio
     *
     * @return self
     */
    public function setFolio($Folio)
    {
    	$this->Folio = $Folio;
    	$this->data['Folio'] = $Folio;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getFecha()
    {
    	return $this->Fecha;
    }

    /**
     * @param mixed $Fecha
     *
     * @return self
     */
    public function setFecha($Fecha)
    {
    	$this->Fecha = $Fecha;
    	$this->data['Fecha'] = $Fecha;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getFormaPago()
    {
    	return $this->FormaPago;
    }

    /**
     * @param mixed $FormaPago
     *
     * @return self
     */
    public function setFormaPago($FormaPago)
    {
    	$this->FormaPago = $FormaPago;
    	$this->data['FormaPago'] = $FormaPago;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getCondicionesDePago()
    {
    	return $this->CondicionesDePago;
    }

    /**
     * @param mixed $CondicionesDePago
     *
     * @return self
     */
    public function setCondicionesDePago($CondicionesDePago)
    {
    	$this->CondicionesDePago = $CondicionesDePago;
    	$this->data['CondicionesDePago'] = $CondicionesDePago;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getSubTotal()
    {
    	return $this->SubTotal;
    }

    /**
     * @param mixed $SubTotal
     *
     * @return self
     */
    public function setSubTotal($SubTotal)
    {
    	$this->SubTotal = $SubTotal;
    	$this->data['SubTotal'] = number_format($SubTotal, 2, '.', '');
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getDescuento()
    {
    	return $this->Descuento;
    }

    /**
     * @param mixed $Descuento
     *
     * @return self
     */
    public function setDescuento($Descuento)
    {
    	$this->Descuento = $Descuento;
    	$this->data['Descuento'] = number_format($Descuento, 2, '.', '');
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getMoneda()
    {
    	return $this->Moneda;
    }

    /**
     * @param mixed $Moneda
     *
     * @return self
     */
    public function setMoneda($Moneda)
    {
    	$this->Moneda = $Moneda;
    	$this->data['Moneda'] = $Moneda;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getTipoCambio()
    {
    	return $this->TipoCambio;
    }

    /**
     * @param mixed $TipoCambio
     *
     * @return self
     */
    public function setTipoCambio($TipoCambio)
    {
    	$this->TipoCambio = $TipoCambio;
    	$this->data['TipoCambio'] = $TipoCambio;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getTotal()
    {
    	return $this->Total;
    }

    /**
     * @param mixed $Total
     *
     * @return self
     */
    public function setTotal($Total)
    {
    	$this->Total = $Total;
    	$this->data['Total'] = number_format($Total, 2, '.', '');
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getTipoDeComprobante()
    {
    	return $this->TipoDeComprobante;
    }

    /**
     * @param mixed $TipoDeComprobante
     *
     * @return self
     */
    public function setTipoDeComprobante($TipoDeComprobante)
    {
    	$this->TipoDeComprobante = $TipoDeComprobante;
    	$this->data['TipoDeComprobante'] = $TipoDeComprobante;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getMetodoPago()
    {
    	return $this->MetodoPago;
    }

    /**
     * @param mixed $MetodoPago
     *
     * @return self
     */
    public function setMetodoPago($MetodoPago)
    {
    	$this->MetodoPago = $MetodoPago;
    	$this->data['MetodoPago'] = $MetodoPago;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getLugarExpedicion()
    {
    	return $this->LugarExpedicion;
    }

    /**
     * @param mixed $LugarExpedicion
     *
     * @return self
     */
    public function setLugarExpedicion($LugarExpedicion)
    {
    	$this->LugarExpedicion = $LugarExpedicion;
    	$this->data['LugarExpedicion'] = $LugarExpedicion;
    	return $this;
    }

    /**
     * @return mixed
     */
    public function getLeyendaFolio()
    {
    	return $this->LeyendaFolio;
    }

    /**
     * @param mixed $LeyendaFolio
     *
     * @return self
     */
    public function setLeyendaFolio($LeyendaFolio)
    {
    	$this->LeyendaFolio = $LeyendaFolio;
    	$this->data['LeyendaFolio'] = $LeyendaFolio;
    	return $this;
    }

    public function setReceptor($receptor)
    {
    	$this->data['Receptor'] = $receptor;
    }

    public function setConceptos($conceptos)
    {
    	$this->data['Conceptos'] = $conceptos;
    }

    public function setImpuestos($impuestos)
    {
    	$this->data['Impuestos'] = $impuestos;
    }

    public function setEmisor($emisor)
    {
    	$this->data['Emisor'] = $emisor;
    }

}
